Package: RcppML
Type: Package
Title: Rcpp Machine Learning Library
Version: 0.3.7.1
Date: 2021-09-21
Authors@R: person("Zachary", "DeBruine", 
    email = "zacharydebruine@gmail.com", 
    role = c("aut", "cre"), 
    comment = c(ORCID = "0000-0003-2234-4827"))
Description: Fast machine learning algorithms including matrix factorization 
    and divisive clustering for large sparse and dense matrices.
License: GPL (>= 2)
Imports: Rcpp, Matrix, methods, stats
LinkingTo: Rcpp, RcppEigen
VignetteBuilder: knitr
RoxygenNote: 7.1.1
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
URL: https://github.com/zdebruine/RcppML
BugReports: https://github.com/zdebruine/RcppML/issues
NeedsCompilation: yes
Packaged: 2026-03-09 07:57:53 UTC; hornik
Author: Zachary DeBruine [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2234-4827>)
Maintainer: Zachary DeBruine <zacharydebruine@gmail.com>
Repository: CRAN
Date/Publication: 2026-03-09 08:16:34 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 00:46:23 UTC; unix
Archs: RcppML.so.dSYM
