useDynLib(RcppML)
import(Rcpp)
import(Matrix)
importFrom("methods", "as", "is", "canCoerce", "new")
importFrom("stats", "runif")
export(nnls)
export(project)
export(mse)
export(nmf)
export(bipartition)
export(dclust)
export(setRcppMLthreads)
export(getRcppMLthreads)