#' RcppML: Rcpp Machine Learning Library
#'
#' @description
#' High-performance non-negative matrix factorization and linear model projection for sparse matrices, and fast non-negative least squares implementations
#'
#' @import knitr Matrix RcppEigen
#' @importFrom Rcpp evalCpp
#' @importFrom methods as canCoerce
#' @useDynLib RcppML, .registration = TRUE
#' @docType package
#' @name RcppML
#' @author Zach DeBruine
#' @aliases RcppML-package
#' @md
#'
NULL